#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Drawing::Imaging;



namespace FilterTemplate {

	public ref class SettingsForm : public System::Windows::Forms::Form
	{
	public:
		filtersettings* FilterSetting;
		bool      FilterCancelled, init;
		Bitmap^   PreviewBitmap;
		Rectangle SourceRect, DestinationRect;
		int       SWidth, SHeight;
		int       LastX, LastY;
		bool      MouseButtonDown;

	private: 
		System::Windows::Forms::Button^			ButtonApply;
		System::Windows::Forms::Button^			ButtonCancel;
		System::Windows::Forms::HScrollBar^		ScrollBarIndexRemain;
		System::Windows::Forms::HScrollBar^		ScrollBarIndexOther;
		System::Windows::Forms::Label^			LabelIndexRemain;
		System::Windows::Forms::Label^			LabelIndexOther;
		System::Windows::Forms::Label^			label2;
		System::Windows::Forms::Label^			label3;
		System::Windows::Forms::PictureBox^		Preview;
		System::Windows::Forms::PictureBox^		PictureBoxNewColor;
		System::Windows::Forms::PictureBox^		PictureBoxRemainColor;
		System::ComponentModel::IContainer^		components;

	public:
		SettingsForm(filtersettings* fs)
		{
			init = true;
			InitializeComponent();

			FilterSetting = fs;

			ScrollBarIndexOther->Minimum  = 0;
			ScrollBarIndexOther->Maximum  = 255;
			ScrollBarIndexRemain->Minimum = 0;
			ScrollBarIndexRemain->Maximum = 255;
			ScrollBarIndexOther->Value    = fs->index_other;
			ScrollBarIndexRemain->Value   = fs->index_remain;

			SWidth = fs->width;
			SHeight = fs->height;

			FilterCancelled = true;

			PreviewBitmap = gcnew Bitmap(Preview->ClientSize.Width, Preview->ClientSize.Height, PixelFormat::Format8bppIndexed);
			ColorPalette^ cPal = PreviewBitmap->Palette;

			for (int i = 0; i < 256; i++) 
			{
				int red   = ((unsigned char*)(fs->palette256 + i))[2];
				int green = ((unsigned char*)(fs->palette256 + i))[1];
				int blue  = ((unsigned char*)(fs->palette256 + i))[0];
				cPal->Entries[i] = Color::FromArgb(red, green, blue);
			}
			PreviewBitmap->Palette = cPal;

			DestinationRect.X = (SWidth < Preview->ClientSize.Width) ? ((Preview->ClientSize.Width - SWidth) / 2) : 0;
			DestinationRect.Y = (SHeight < Preview->ClientSize.Height) ? ((Preview->ClientSize.Height - SHeight) / 2) : 0;
			DestinationRect.Width = (DestinationRect.X) ? SWidth : Preview->ClientSize.Width;
			DestinationRect.Height = (DestinationRect.Y) ? SHeight : Preview->ClientSize.Height;

			SourceRect.X = 0;
			SourceRect.Y = 0;
			SourceRect.Width = DestinationRect.Width;   // SHOULD BE EQUAL !!!
			SourceRect.Height = DestinationRect.Height; // SHOULD BE EQUAL !!!

			init = false;

			Scroll_ValueChanged(nullptr,nullptr);

			MouseButtonDown = false;
		}

	protected:

		~SettingsForm()
		{
			if (components)
			{
				delete components;
			}
		}


#pragma region Windows Form Designer generated code

		void InitializeComponent(void)
		{
			this->Preview = (gcnew System::Windows::Forms::PictureBox());
			this->ButtonApply = (gcnew System::Windows::Forms::Button());
			this->ButtonCancel = (gcnew System::Windows::Forms::Button());
			this->ScrollBarIndexRemain = (gcnew System::Windows::Forms::HScrollBar());
			this->label2 = (gcnew System::Windows::Forms::Label());
			this->LabelIndexRemain = (gcnew System::Windows::Forms::Label());
			this->LabelIndexOther = (gcnew System::Windows::Forms::Label());
			this->label3 = (gcnew System::Windows::Forms::Label());
			this->ScrollBarIndexOther = (gcnew System::Windows::Forms::HScrollBar());
			this->PictureBoxNewColor = (gcnew System::Windows::Forms::PictureBox());
			this->PictureBoxRemainColor = (gcnew System::Windows::Forms::PictureBox());
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Preview))->BeginInit();
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->PictureBoxNewColor))->BeginInit();
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->PictureBoxRemainColor))->BeginInit();
			this->SuspendLayout();
			// 
			// Preview
			// 
			this->Preview->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->Preview->Cursor = System::Windows::Forms::Cursors::NoMove2D;
			this->Preview->Location = System::Drawing::Point(12, 12);
			this->Preview->Name = L"Preview";
			this->Preview->Size = System::Drawing::Size(154, 154);
			this->Preview->TabIndex = 0;
			this->Preview->TabStop = false;
			this->Preview->MouseMove += gcnew System::Windows::Forms::MouseEventHandler(this, &SettingsForm::Preview_MouseMove);
			this->Preview->MouseDown += gcnew System::Windows::Forms::MouseEventHandler(this, &SettingsForm::Preview_MouseDown);
			this->Preview->Paint += gcnew System::Windows::Forms::PaintEventHandler(this, &SettingsForm::Preview_OnPaint);
			this->Preview->MouseUp += gcnew System::Windows::Forms::MouseEventHandler(this, &SettingsForm::Preview_MouseUp);
			// 
			// ButtonApply
			// 
			this->ButtonApply->Location = System::Drawing::Point(183, 143);
			this->ButtonApply->Name = L"ButtonApply";
			this->ButtonApply->Size = System::Drawing::Size(75, 23);
			this->ButtonApply->TabIndex = 2;
			this->ButtonApply->Text = L"Apply";
			this->ButtonApply->UseVisualStyleBackColor = true;
			this->ButtonApply->Click += gcnew System::EventHandler(this, &SettingsForm::ButtonApply_Click);
			// 
			// ButtonCancel
			// 
			this->ButtonCancel->Location = System::Drawing::Point(264, 143);
			this->ButtonCancel->Name = L"ButtonCancel";
			this->ButtonCancel->Size = System::Drawing::Size(75, 23);
			this->ButtonCancel->TabIndex = 3;
			this->ButtonCancel->Text = L"Cancel";
			this->ButtonCancel->UseVisualStyleBackColor = true;
			this->ButtonCancel->Click += gcnew System::EventHandler(this, &SettingsForm::ButtonCancel_Click);
			// 
			// ScrollBarIndexRemain
			// 
			this->ScrollBarIndexRemain->LargeChange = 1;
			this->ScrollBarIndexRemain->Location = System::Drawing::Point(206, 39);
			this->ScrollBarIndexRemain->Maximum = 255;
			this->ScrollBarIndexRemain->Name = L"ScrollBarIndexRemain";
			this->ScrollBarIndexRemain->Size = System::Drawing::Size(250, 17);
			this->ScrollBarIndexRemain->TabIndex = 8;
			this->ScrollBarIndexRemain->ValueChanged += gcnew System::EventHandler(this, &SettingsForm::Scroll_ValueChanged);
			this->ScrollBarIndexRemain->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &SettingsForm::Scroll_Change);
			// 
			// label2
			// 
			this->label2->AutoSize = true;
			this->label2->ForeColor = System::Drawing::Color::White;
			this->label2->Location = System::Drawing::Point(203, 26);
			this->label2->Name = L"label2";
			this->label2->Size = System::Drawing::Size(119, 13);
			this->label2->TabIndex = 11;
			this->label2->Text = L"Ez az index marad meg:";
			// 
			// LabelIndexRemain
			// 
			this->LabelIndexRemain->AutoSize = true;
			this->LabelIndexRemain->ForeColor = System::Drawing::Color::White;
			this->LabelIndexRemain->Location = System::Drawing::Point(459, 43);
			this->LabelIndexRemain->Name = L"LabelIndexRemain";
			this->LabelIndexRemain->Size = System::Drawing::Size(13, 13);
			this->LabelIndexRemain->TabIndex = 14;
			this->LabelIndexRemain->Text = L"0";
			// 
			// LabelIndexOther
			// 
			this->LabelIndexOther->AutoSize = true;
			this->LabelIndexOther->ForeColor = System::Drawing::Color::White;
			this->LabelIndexOther->Location = System::Drawing::Point(459, 90);
			this->LabelIndexOther->Name = L"LabelIndexOther";
			this->LabelIndexOther->Size = System::Drawing::Size(13, 13);
			this->LabelIndexOther->TabIndex = 17;
			this->LabelIndexOther->Text = L"0";
			// 
			// label3
			// 
			this->label3->AutoSize = true;
			this->label3->ForeColor = System::Drawing::Color::White;
			this->label3->Location = System::Drawing::Point(203, 73);
			this->label3->Name = L"label3";
			this->label3->Size = System::Drawing::Size(227, 13);
			this->label3->TabIndex = 16;
			this->label3->Text = L"Minden ms indexet erre az indexre cserljnk:";
			// 
			// ScrollBarIndexOther
			// 
			this->ScrollBarIndexOther->LargeChange = 1;
			this->ScrollBarIndexOther->Location = System::Drawing::Point(206, 86);
			this->ScrollBarIndexOther->Maximum = 255;
			this->ScrollBarIndexOther->Name = L"ScrollBarIndexOther";
			this->ScrollBarIndexOther->Size = System::Drawing::Size(250, 17);
			this->ScrollBarIndexOther->TabIndex = 15;
			this->ScrollBarIndexOther->ValueChanged += gcnew System::EventHandler(this, &SettingsForm::Scroll_ValueChanged);
			this->ScrollBarIndexOther->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &SettingsForm::Scroll_Change);
			// 
			// PictureBoxNewColor
			// 
			this->PictureBoxNewColor->BackColor = System::Drawing::Color::White;
			this->PictureBoxNewColor->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;
			this->PictureBoxNewColor->Location = System::Drawing::Point(172, 86);
			this->PictureBoxNewColor->Name = L"PictureBoxNewColor";
			this->PictureBoxNewColor->Size = System::Drawing::Size(31, 17);
			this->PictureBoxNewColor->TabIndex = 24;
			this->PictureBoxNewColor->TabStop = false;
			// 
			// PictureBoxRemainColor
			// 
			this->PictureBoxRemainColor->BackColor = System::Drawing::Color::White;
			this->PictureBoxRemainColor->BorderStyle = System::Windows::Forms::BorderStyle::FixedSingle;
			this->PictureBoxRemainColor->Location = System::Drawing::Point(172, 39);
			this->PictureBoxRemainColor->Name = L"PictureBoxRemainColor";
			this->PictureBoxRemainColor->Size = System::Drawing::Size(31, 17);
			this->PictureBoxRemainColor->TabIndex = 23;
			this->PictureBoxRemainColor->TabStop = false;
			// 
			// SettingsForm
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->BackColor = System::Drawing::Color::SteelBlue;
			this->ClientSize = System::Drawing::Size(484, 178);
			this->Controls->Add(this->PictureBoxNewColor);
			this->Controls->Add(this->PictureBoxRemainColor);
			this->Controls->Add(this->LabelIndexOther);
			this->Controls->Add(this->label3);
			this->Controls->Add(this->ScrollBarIndexOther);
			this->Controls->Add(this->LabelIndexRemain);
			this->Controls->Add(this->label2);
			this->Controls->Add(this->ScrollBarIndexRemain);
			this->Controls->Add(this->ButtonCancel);
			this->Controls->Add(this->ButtonApply);
			this->Controls->Add(this->Preview);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->MaximizeBox = false;
			this->MinimizeBox = false;
			this->Name = L"SettingsForm";
			this->ShowIcon = false;
			this->ShowInTaskbar = false;
			this->Text = L"MaskColor";
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Preview))->EndInit();
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->PictureBoxNewColor))->EndInit();
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->PictureBoxRemainColor))->EndInit();
			this->ResumeLayout(false);
			this->PerformLayout();

		}
#pragma endregion

	public:
		void RefreshPreview(bool applyfilter)
		{
			if (init) return;

			BitmapData^ bd = PreviewBitmap->LockBits( Rectangle(0, 0, PreviewBitmap->Width, PreviewBitmap->Height), ImageLockMode::ReadWrite, PixelFormat::Format8bppIndexed);
			FilterSetting->strideout = bd->Stride;
			FilterSetting->dataout = (Byte*)(void*)bd->Scan0;

			if (applyfilter)
				FilterSpace::ImageFilter::FilterCore(FilterSetting, SourceRect.X, SourceRect.Y, SourceRect.Width, SourceRect.Height);
			else
				FilterSpace::ImageFilter::ClipImage( FilterSetting, SourceRect.X, SourceRect.Y, SourceRect.Width, SourceRect.Height);

			PreviewBitmap->UnlockBits(bd);
			this->Refresh();
		}

	private: 
		System::Void ButtonApply_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			FilterCancelled = false;
			this->Close();
		}

		System::Void ButtonCancel_Click(System::Object^  sender, System::EventArgs^  e) { this->Close(); }

		System::Void Preview_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (e->Button == System::Windows::Forms::MouseButtons::Left) 
			{
				MouseButtonDown = true;
				LastX = e->X;
				LastY = e->Y;
				RefreshPreview(false);
			}
		}

		System::Void Preview_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (!MouseButtonDown) return;
			if (!DestinationRect.X)
			{
				SourceRect.X -= e->X - LastX;
				if (SourceRect.X < 0) SourceRect.X = 0;
				if (SourceRect.X >= SWidth - DestinationRect.Width) SourceRect.X = SWidth - DestinationRect.Width - 1;
			}
			if (!DestinationRect.Y)
			{
				SourceRect.Y -= e->Y - LastY;
				if (SourceRect.Y < 0) SourceRect.Y = 0;
				if (SourceRect.Y >= SHeight - DestinationRect.Height) SourceRect.Y = SHeight - DestinationRect.Height - 1;
			}
			RefreshPreview(false);
			LastX = e->X;
			LastY = e->Y;
		}

		System::Void Preview_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (e->Button == System::Windows::Forms::MouseButtons::Left) MouseButtonDown = false;
			if (!MouseButtonDown) RefreshPreview(true);
		}

		System::Void Preview_OnPaint(System::Object^  sender, System::Windows::Forms::PaintEventArgs^  e) 
		{
			Graphics^ g = e->Graphics;
			Rectangle r = Rectangle(0, 0, DestinationRect.Width, DestinationRect.Height);
			g->DrawImage(PreviewBitmap, DestinationRect, r, GraphicsUnit::Pixel);
		}

		System::Void Scroll_Change(System::Object^  sender, System::Windows::Forms::ScrollEventArgs^  e) 
		{
			if (!init)
			{
				FilterSetting->index_other  = ScrollBarIndexOther->Value;
				FilterSetting->index_remain = ScrollBarIndexRemain->Value;
			}
			LabelIndexOther->Text       = ScrollBarIndexOther->Value.ToString();
			LabelIndexRemain->Text      = ScrollBarIndexRemain->Value.ToString();

			int r, g, b, index;

			index = FilterSetting->index_remain;
			b = ((byte*)(FilterSetting->palette256 + index))[0];
			g = ((byte*)(FilterSetting->palette256 + index))[1];
			r = ((byte*)(FilterSetting->palette256 + index))[2];
			PictureBoxRemainColor->BackColor     = Color::FromArgb(r, g, b);

			index = FilterSetting->index_other;
			b = ((byte*)(FilterSetting->palette256 + index))[0];
			g = ((byte*)(FilterSetting->palette256 + index))[1];
			r = ((byte*)(FilterSetting->palette256 + index))[2];
			PictureBoxNewColor->BackColor     = Color::FromArgb(r, g, b);

			RefreshPreview(true);
		}

		System::Void Scroll_ValueChanged(System::Object^  sender, System::EventArgs^  e) { Scroll_Change(nullptr, nullptr); }
};
}
